/* Solaris wrappers for the nss_db module.
   Copyright (C) 2011 Mark R. Bannister <cambridge@users.sourceforge.net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdlib.h>
#include <strings.h>

#include "db-sol-common.h"
#include "db-syslog.h"

nss_backend_t *
_nss_db_constr (ops, n_ops)
	nss_backend_op_t ops[];
	int n_ops;
{
    nss_backend_t *be;

    if (!(be=malloc(sizeof(nss_backend_t)))) return NULL;
    be->ops = ops;
    be->n_ops = n_ops;
    return be;
}

nss_status_t
_nss_db_destr (be, dummy)
	nss_backend_t *be;
	void *dummy;
{
    if (be) free(be);
    return NSS_SUCCESS;
}

nss_status_t
_nss_db_str2ent (result, args)
	void *result;
	nss_XbyY_args_t *args;
{
    nss_status_t ret;
    char *str;
    int parsestat, linelen;
    int (*func)();

    if ((str = strdup (args->buf.buffer)))
    {
	func = args->str2ent;
	linelen = strlen (str);
	parsestat = (*func)(str, linelen, result,
				    args->buf.buffer, args->buf.buflen);

	if (parsestat == NSS_STR_PARSE_SUCCESS)
	{
	    args->returnval = result ? result : args->buf.buffer;
	    args->returnlen = linelen;
	    ret = NSS_SUCCESS;
	}
	else if (parsestat == NSS_STR_PARSE_ERANGE)
	{
	    args->erange = 1;
	    ret = NSS_NOTFOUND;
	}
	else ret = NSS_NOTFOUND;

	free (str);
    }
    else ret = NSS_UNAVAIL;

    return ret;
}

int
_nss_db_str2ent_grp (line, grp, buffer, buflen, errnop, str2ent)
	char *line;
	struct group *grp;
	void *buffer;
	size_t buflen;
	int *errnop;
	nss_str2ent_t str2ent;
{
    int parsestat;

    syslog (LOG_DEBUG, PACKAGE ": _nss_db_str2ent_grp (%s)\n", line);

    parsestat = (*str2ent)(line, strlen(line), grp, buffer, buflen);

    if (parsestat == NSS_STR_PARSE_SUCCESS)
    {
	syslog (LOG_DEBUG, PACKAGE ": str2ent () returned gid %ld\n",
								grp->gr_gid);
	return 1;
    }

    if (parsestat == NSS_STR_PARSE_ERANGE) *errnop=ERANGE;
    return 0;
}
