/* Solaris wrappers for the nss_db module.
   Copyright (C) 2011 Mark R. Bannister <cambridge@users.sourceforge.net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdlib.h>
#include <strings.h>

#include "db-syslog.h"
#include "db-sol-common.h"
#include "nss_db.h"

static nss_status_t
_nss_db_shadow_endspent_solwrap (dummy1, dummy2)
	void *dummy1, *dummy2;
{
    return (nss_status_t) _nss_db_endspent();
}

static nss_status_t
_nss_db_shadow_setspent_solwrap (dummy1, dummy2)
	void *dummy1, *dummy2;
{
    return (nss_status_t) _nss_db_setspent(0);
}

static nss_status_t
_nss_db_shadow_getspent_solwrap (be, a)
	nss_backend_t *be;
	void *a;
{
    nss_XbyY_args_t *args = (nss_XbyY_args_t *)a;
    enum nss_status ret;
    void *result = args->buf.result;

    ret = _nss_db_getspent_r (result, args->buf.buffer,
				args->buf.buflen, &args->erange);

    if (ret == NSS_STATUS_BUFFER)
	ret = (enum nss_status)_nss_db_str2ent (result, args);
    else if (ret == NSS_STATUS_SUCCESS)
	args->returnval = result;

    return (nss_status_t)ret;
}

static nss_status_t
_nss_db_shadow_getspnam_solwrap (be, a)
	nss_backend_t *be;
	void *a;
{
    nss_XbyY_args_t *args = (nss_XbyY_args_t *)a;
    enum nss_status ret;
    void *result = args->buf.result;

    ret = _nss_db_getspnam_r (args->key.name, result, args->buf.buffer,
				args->buf.buflen, &args->erange);

    if (ret == NSS_STATUS_BUFFER)
	ret = (enum nss_status)_nss_db_str2ent (result, args);
    else if (ret == NSS_STATUS_SUCCESS)
	args->returnval = result;

    return (nss_status_t)ret;
}

static nss_backend_op_t shadow_ops[] =
{
    _nss_db_destr,
    _nss_db_shadow_endspent_solwrap,	/* NSS_DBOP_ENDENT */
    _nss_db_shadow_setspent_solwrap,	/* NSS_DBOP_SETENT */
    _nss_db_shadow_getspent_solwrap,	/* NSS_DBOP_GETENT */
    _nss_db_shadow_getspnam_solwrap	/* NSS_DBOP_SHADOW_BYNAME */
};

nss_backend_t *
_nss_db_shadow_constr (dummy1, dummy2, dummy3)
	const char *dummy1, *dummy2, *dummy3;
{
    syslog (LOG_DEBUG, PACKAGE ": _nss_db_shadow_constr () called\n");
    return _nss_db_constr (shadow_ops,
		sizeof (shadow_ops) / sizeof (shadow_ops[0]));
}
