/* Common database open/close routines for nss_db.
   Copyright (C) 1999, 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef _NSS_DB_H
#define _NSS_DB_H	1

#include <config.h>
#include <db.h>
#include <limits.h>
#ifdef HAVE_NSS_H
#  include <nss.h>
#endif

#ifdef HAVE_NSS_COMMON_H

   /* Cater for some differences on Solaris */
#  include <nss_common.h>

   enum nss_status {
	NSS_STATUS_SUCCESS = 0,
	NSS_STATUS_NOTFOUND = 1,
	NSS_STATUS_UNAVAIL = 2,
	NSS_STATUS_TRYAGAIN = 3,
	NSS_STATUS_RETURN = 100,
	NSS_STATUS_BUFFER = 101,
   };

#endif

#ifdef SOLARIS_NSS
#  define EXTRA_INITGROUP_ARGS ,nss_str2ent_t str2ent
#else
#  define EXTRA_INITGROUP_ARGS
#endif

#ifndef NSS_LINELEN_GROUP
#  define NSS_LINELEN_GROUP 1024
#endif

#ifndef NSS_BUFLEN_GROUP
#  define NSS_BUFLEN_GROUP NSS_LINELEN_GROUP
#endif

/* Open the database stored in FILE.  If succesful, store the database
   handle in *DBP and return NSS_STATUS_SUCCESS.  On failure, return
   the appropriate lookup status.  */
extern enum nss_status internal_setent (const char *file, DB **dbp, DB_ENV **dbenvp);

/* Close the database *DBP.  */
extern void internal_endent (DB **dbp, DB_ENV **dbenvp);

/* The following are defined in glibc in include/alloca.h.  */
extern int __libc_alloca_cutoff (size_t size) __attribute__ ((const));
/* #define PTHREAD_STACK_MIN 16384 */
#define __libc_use_alloca(size) \
  (__builtin_expect ((size) <= PTHREAD_STACK_MIN / 4, 1)                     \
                    || __libc_alloca_cutoff (size))
#if defined __i386__ || defined __x86_64__
# define extend_alloca(buf, len, newlen) \
  (__typeof (buf)) ({ size_t __newlen = (newlen);                            \
                      char *__newbuf = alloca (__newlen);                    \
                      if (__newbuf + __newlen == (char *) buf)               \
                        len += __newlen;                                     \
                      else                                                   \
                        len = __newlen;                                      \
                      __newbuf; })
#else
# define extend_alloca(buf, len, newlen) \
  alloca (((len) = (newlen)))
#endif

#endif	/* nss_db.h */
